/**
 * srvoa - soa infrastructure for node js
 *
 * @copyright   Copyright (c) 2015, Alrik Zachert
 * @license     https://gitlab.com/a-z/node-srvoa/blob/master/LICENSE BSD-2-Clause
 */

"use strict";

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

var _get = function get(_x, _x2, _x3) { var _again = true; _function: while (_again) { var object = _x, property = _x2, receiver = _x3; desc = parent = getter = undefined; _again = false; if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { _x = parent; _x2 = property; _x3 = receiver; _again = true; continue _function; } } else if ('value' in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } } };

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

function _inherits(subClass, superClass) { if (typeof superClass !== 'function' && superClass !== null) { throw new TypeError('Super expression must either be null or a function, not ' + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var fs = require('fs'),
    Service = require('./Service');

/**
 * Calculate the type of a value - used by recursive merge logic.
 *
 * @private
 * @param value
 * @returns {string}
 */
function getConfigType(value) {
    var type = typeof value;

    if (type === 'object') {
        if (Array.isArray(value)) {
            type = 'array';
        }
    }

    return type;
}

/**
 * Recursively merge object src into dest. Array values won't be merged but overridden.
 *
 * @param dest
 * @param src
 * @returns {*}
 */
function mergeConfig(dest, src) {
    var property, value, destType, srcType;

    for (property in src) {
        value = src[property];

        if (!(property in dest)) {
            dest[property] = value;
        } else if (value === null) {
            delete dest[property];
        } else {
            destType = getConfigType(dest[property]);
            srcType = getConfigType(value);

            if (destType === 'object' && srcType === 'object') {
                mergeConfig(dest[property], value);
            } else {
                dest[property] = value;
            }
        }
    }

    return dest;
}

/**
 * The srvoa application class.
 */

var ConfigService = (function (_Service) {
    _inherits(ConfigService, _Service);

    /**
     * The files to read and merge the config from.
     *
     * @property files {Array}
     */

    /**
     * The config hash.
     *
     * @property config {Object}
     */

    /**
     * Initialize the config service.
     *
     * @constructor
     */

    function ConfigService(serviceManager) {
        _classCallCheck(this, ConfigService);

        _get(Object.getPrototypeOf(ConfigService.prototype), 'constructor', this).call(this, serviceManager);

        this.config = {};
        this.files = [];
    }

    /**
     * @param   {Object} config
     * @return  {ConfigService}
     */

    _createClass(ConfigService, [{
        key: 'configure',
        value: function configure(config) {
            if (typeof config.files !== 'undefined') {
                this.setFiles(config.files);
            }

            return this;
        }

        /**
         * @inheritDoc
         */
    }, {
        key: 'launch',
        value: function launch() {
            this.readConfig();

            return this;
        }

        /**
         * Return the list of files to read the config from.
         *
         * @return {Array}
         */
    }, {
        key: 'getFiles',
        value: function getFiles() {
            return this.files;
        }

        /**
         * Set the files to read the config from.
         *
         * @param   {Array} files
         * @returns {ConfigService}
         */
    }, {
        key: 'setFiles',
        value: function setFiles(files) {
            this.files = files;

            return this;
        }

        /**
         * Read the config files (if existent) and recursively merge the settings.
         * The reader also adds support for simple comments in the config.json files.
         *
         * @return  {ConfigService}
         */
    }, {
        key: 'readConfig',
        value: function readConfig() {
            var files = this.files,
                i = 0,
                l = files.length,
                file,
                data;

            this.config = {};

            for (; i < l; ++i) {
                file = files[i];

                if (fs.existsSync(file)) {
                    data = require(file);

                    mergeConfig(this.config, data);
                }
            }

            return this;
        }

        /**
         * @return {Object}
         */
    }, {
        key: 'getConfig',
        value: function getConfig() {
            return Object(this.config);
        }

        /**
         * Sets and merges multiple config sets.
         *
         * @param   {...Object} config
         * @return  {ConfigService}
         */
    }, {
        key: 'setConfig',
        value: function setConfig(config) {
            var l = arguments.length;

            if (l === 1) {
                this.config = arguments[0];
            } else {
                this.config = {};

                for (var i = 0; i < l; ++i) {
                    mergeConfig(this.config, arguments[i]);
                }
            }

            return this;
        }

        /**
         * @param   {String} key
         * @param   {*} defaultValue
         * @returns {*}
         */
    }, {
        key: 'get',
        value: function get(key, defaultValue) {
            var parts = key.split('.'),
                l = parts.length,
                i = 0,
                scopeKey,
                scope = this.config;

            try {
                for (; i < l; ++i) {
                    scopeKey = parts[i];

                    if (scopeKey in scope) {
                        scope = scope[scopeKey];
                    } else {
                        return defaultValue;
                    }
                }

                return scope;
            } catch (e) {
                return defaultValue;
            }
        }
    }]);

    return ConfigService;
})(Service);

module.exports = ConfigService;